#
#  FILE NAME:
#      sequence_helpers.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#
## @package p3.base.sequence_helpers
#  General purpose sequence utilities and operations.

# Standard library imports:
import logging


_LOG = logging.getLogger( "p3.base" )


## Check if sequence contains duplicates.
#
#  The function searches for duplicates between items produced by the value_adapter_cb routine from
#  the input sequence items.
#
#  \return True when duplicates were not found in the input sequence and False otherwise.
#  \param seq The sequence of input values.
#  \param value_adapter_cb Optional callback used to adapt input items for duplicate detection. It
#      should accept input sequence item as its only attribute.
#
#  Examples of usage:
#  \li Detection of duplicate characters:
#  \code
# if not uniqueness_check( "ABA" ):
#     print "Duplicates found, sorry."
#  \endcode
#  \li Detection of case insensitive duplicates in a sequence of strings:
#  \code
# if uniqueness_check( strings, lambda v: v.upper() ):
#     print "No duplicates found."
#  \endcode
#  \li Detection of duplicate objects basing on one of their member variables:
#  \code
# class Cont( object ):
#     def __init__( self, name ):
#         self.name = name
#
# assert uniqueness_check( conts, lambda o: o.name ), "Duplicates found!"
#  \endcode
def uniqueness_check( seq, value_adapter_cb=None ):
    assert value_adapter_cb is None or callable(value_adapter_cb)

    if value_adapter_cb is None:
        unique = set( seq )
    else:
        unique = set( value_adapter_cb(v) for v in seq )

    return len(unique) == len(seq)


## Extract duplicates from sequence and return them as a tuple.
#
#  The function searches for duplicates between items produced by the value_adapter_cb callback
#  from the input sequence items but returns list of unprocessed values. Two identical input values
#  will be represented by a single item in the output tuple. Two different input values resulting
#  in the same value after value_adapter_cb transformation will be represented in the output tuple
#  by their two input values.
#
#  \return Tuple of duplicate, unprocessed sequence items. The tuple items are guaranteed to be
#      unique but processing them using the value_adapter_cb callback may produce some duplicates.
#
#  \param seq Sequence of input items.
#  \param value_adapter_cb Optional callback used to adapt input items for duplicate detection. It
#      should accept input sequence item as its only attribute.
#  \see uniqueness_check function documentation for few examples of the value_adapter_cb usage.
def get_duplicates( seq, value_adapter_cb=None ):
    value_map = {}

    for value in seq:
        adapted = (value if value_adapter_cb is None else value_adapter_cb(value))

        value_map.setdefault(adapted,[]).append(value)

    result = set()

    _LOG.info( value_map )

    for values in value_map.values():
        if len(values) > 1:
            result.update( set(values) )

    return tuple(result)
