#
#  FILE NAME:
#      check.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import safe_fmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.execute import exec_query


_LOG = logging.getLogger( "p3.db" )


def db_exists( mng_conn, db_name ):
    assert isinstance( db_name, str )

    if mng_conn.dbms == DBMS.mysql:
        result = my_sql_db_exists( mng_conn, db_name )
    elif mng_conn.dbms == DBMS.pgsql:
        result = pg_sql_db_exists( mng_conn, db_name )
    else:
        msg = safe_fmt( "Unsupported database management system: '%s'",
                        '/'.join( DBMS.get_names( mng_conn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbms_not_supported, msg )

    _LOG.info( "Database '%s' %s exists...", db_name, ("" if result else "doesn't") )

    return result


def my_sql_db_exists( mng_conn, db_name ):
    query = \
        "SELECT schema_name FROM information_schema.schemata WHERE (schema_name = %s);"
    return bool(exec_query( mng_conn, query, (db_name,) ).rows)


def pg_sql_db_exists( mng_conn, db_name ):
    query = \
        "SELECT datname FROM pg_database WHERE (datname = %s);"
    return bool(exec_query( mng_conn, query, (db_name,) ).rows)


def table_exists( db_conn, table_name ):
    assert isinstance( table_name, str )

    if db_conn.dbms == DBMS.mysql:
        result = my_sql_table_exists( db_conn, table_name )
    elif db_conn.dbms == DBMS.pgsql:
        result = pg_sql_table_exists( db_conn, table_name )
    else:
        msg = safe_fmt( "Unsupported database management system: '%s'",
                        '/'.join( DBMS.get_names( db_conn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbms_not_supported, msg )

    _LOG.info( "Table '%s.%s' %s exists...", db_conn.db_name, table_name,
               ("" if result else "doesn't") )

    return result


def my_sql_table_exists( db_conn, table_name ):
    query = \
        """
        SELECT
            1
        FROM
            information_schema.tables
        WHERE
            (table_schema = %s) AND
            (table_name = %s);
        """

    return bool(exec_query( db_conn, query, (db_conn.db_name, table_name) ).rows)


def pg_sql_table_exists( db_conn, table_name ):
    query = \
        """
        SELECT
            1
        FROM
            pg_tables
        WHERE
            (schemaname = 'public') AND
            (tablename = %s);
        """

    return bool(exec_query( db_conn, query, (table_name, ) ).rows)
