#
#  FILE NAME:
#      date_converter.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import datetime
import logging
import numbers
import re

# P3 imports:
from p3.base import Enum, safe_fmt
from p3.logic.var.converter.conversion_result import ConversionResult

_LOG = logging.getLogger( "p3.logic" )


class DateConverter( object ):
    errors = Enum(
        "empty_input", "invalid_format", "invalid_date", "max_exceeded", "min_exceeded" )
    formats = Enum( "ddmmyy", "ddmmyyyy", "yymmdd", "yyyymmdd" )

    ## ===[ Initialization ]==================================================================== ##

    def __init__( self ):
        self.fmt = self.formats.yyyymmdd
        self.pattern = None
        self.max_val  = None
        self.min_val  = None


    def format( self, fmt ):
        assert fmt in self.formats

        self.fmt = fmt
        return self


    def max( self, val ):
        self.max_val = val
        return self


    def min( self, val ):
        self.min_val = val
        return self


    ## ===[ Conversion ]======================================================================== ##

    def deserialize( self, value ):
        if not isinstance( value, basestring ):
            raise TypeError( safe_fmt( "DateConverter.deserialize method expects basestring as its"
                                       " parameter (got %s instead).", type(value) ) )

        res = ConversionResult( value, self.__class__ )

        if not value:
            res.errors.append( self.errors.empty_input )
            return res

        if self.pattern is None:
            self.pattern = self._build_pattern( self.fmt )
        m = self.pattern.match( value )

        if m is not None:
            if m.group("sep1") != m.group("sep2"):
                res.errors.append( self.errors.invalid_format )
            else:
                if self.fmt in (self.formats.ddmmyy, self.formats.yymmdd):
                    y = 2000+int(m.group("year"))
                else:
                    y = int(m.group("year"))
                try:
                    out_val = datetime.date( y, int(m.group("month")), int(m.group("day")) )
                except ValueError as e:
                    res.errors.append( self.errors.invalid_date )
                else:
                    if (((self.min_val is None) or (out_val >= self.min_val)) and
                        ((self.max_val is None) or (out_val < self.max_val))):
                        res.out_val = out_val
                    elif (self.min_val is not None) and (out_val < self.min_val):
                        res.errors.append( self.errors.min_exceeded )
                    elif self.max_val is not None and out_val >= self.max_val:
                        res.errors.append( self.errors.max_exceeded )
        else:
            res.errors.append( self.errors.invalid_format )

        return res

    @classmethod
    def is_date_like( self, obj ):
        return (hasattr(obj, 'year') and hasattr(obj, 'month') and hasattr(obj, 'day') and
                isinstance(obj.year, numbers.Integral) and
                isinstance(obj.month, numbers.Integral) and
                isinstance(obj.day, numbers.Integral))

    def serialize( self, value ):
        if not self.is_date_like( value ):
            raise TypeError( safe_fmt( "DateConverter.serialize method expects date like object as"
                                       " its parameter (got %s instead).", type(value) ) )

        res = ConversionResult( value, self.__class__ )

        try:
            datetime.date( value.year, value.month, value.day )
        except ValueError as e:
            res.errors.append( self.errors.invalid_date )
        else:
            fmt = { self.formats.ddmmyy:   "{d:02d}/{m:02d}/{y:02d}",
                    self.formats.ddmmyyyy: "{d:02d}/{m:02d}/{y:04d}",
                    self.formats.yymmdd:   "{y:02d}/{m:02d}/{d:02d}",
                    self.formats.yyyymmdd: "{y:04d}/{m:02d}/{d:02d}" }.get( self.fmt )
            assert fmt is not None

            if self.fmt in (self.formats.ddmmyy, self.formats.yymmdd):
                y = value.year % 100
            else:
                y = value.year

            res.out_val = fmt.format( y=y, m=value.month, d=value.day )

        return res


    ## ===[ Internals ]========================================================================= ##

    def _build_pattern( self, fmt ):
        day_grp = "(?P<day>\d{2})"   # day
        mth_grp = "(?P<month>\d{2})" # month
        syr_grp = "(?P<year>\d{2})"  # short year
        lyr_grp = "(?P<year>\d{4})"  # long year
        sep     = "[-/.]{0,1}"       # separator (in future, allowed separators can depend on
                                     # converter configuration)
        sep1 = "(?P<sep1>%s)" % sep
        sep2 = "(?P<sep2>%s)" % sep

        if self.fmt == self.formats.ddmmyy:
            return re.compile( "^%s%s%s%s%s$" % (day_grp,sep1,mth_grp,sep2,syr_grp) )
        elif self.fmt == self.formats.ddmmyyyy:
            return re.compile( "^%s%s%s%s%s$" % (day_grp,sep1,mth_grp,sep2,lyr_grp) )
        elif self.fmt == self.formats.yymmdd:
            return re.compile( "^%s%s%s%s%s$" % (syr_grp,sep1,mth_grp,sep2,day_grp) )
        elif self.fmt == self.formats.yyyymmdd:
            return re.compile( "^%s%s%s%s%s$" % (lyr_grp,sep1,mth_grp,sep2,day_grp) )
