#
#  FILE NAME:
#      env.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import sys
import unittest

# P3 imports:
from p3.conf import configure_logger, module_settings
from p3.test.error import TestModuleError


_LOG = logging.getLogger( "p3.test" )


class TestEnv( object ):
    ## \param settings Either an string defining the settings module path or a Settings like
    #         object.
    def __init__( self, settings, create_suite_cb ):
        assert callable(create_suite_cb)

        self.create_suite_cb = create_suite_cb
        self.options = None

        if isinstance( settings, basestring ):
            _LOG.debug( "Settings provided as a module path." )
            self.settings = module_settings( settings )
        else:
            _LOG.debug( "Settings provided as a Settings object." )
            self.settings = settings

        configure_logger( self.settings )


    def parse_options( self ):
        if sys.version_info >= (2,7):
            return self._parse_options_arg_parse()
        elif sys.version_info >= (2,3):
            return self._parse_options_opt_parse()
        else:
            raise TestModuleError( TestModuleError.codes.unsupported_python_version,
                                   "TestEnv class requires Python 2.3 or newer." )


    def create_selective_suite( self ):
        assert self.options is not None

        return unittest.defaultTestLoader.loadTestsFromNames( self.options.tests )


    def execute( self ):
        self.options = self.parse_options()
        _LOG.info( "Parsed program options: %s", self.options )

        if self.options.tests:
            s = self.create_selective_suite()
        else:
            s = self.create_suite_cb()

        unittest.TextTestRunner( verbosity=self.options.verbosity ).run( s )


    # ===[ Internal ]============================================================================ #

    def _parse_options_arg_parse( self ):
        import argparse

        parser = argparse.ArgumentParser( description="Database upgrade tool test runner." )

        parser.add_argument( "--verbosity",
                             action='store',
                             default=2,
                             type=int,
                             choices=(0,1,2),
                             help="Standard test runner verbosity as expected by unittest."
                             "TextTestRunner class." )

        parser.add_argument( "tests",
                             action='store',
                             type=str,
                             nargs='*',
                             help="Optional list of tests to be executed (if no test is specified"
                             " all tests will be executed)." )

        opts = parser.parse_args()
        return opts


    def _parse_options_opt_parse( self ):
        import optparse

        def verbosity_cb( option, opt_str, value, parser ):
            setattr( parser.values, option.dest, int(value) )

        parser = optparse.OptionParser()
        parser.add_option( "--verbosity",
                           action='callback',
                           dest="verbosity",
                           callback=verbosity_cb,
                           type="choice",
                           choices=("0","1","2"),
                           default=2,
                           help="Standard test runner verbosity as expected by unittest."
                             "TextTestRunner class." )

        options, args = parser.parse_args()
        options.tests = args
        return options
